
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;
import java.util.Scanner;

/**
 * Bowser's Army
 *
 * For the UMD High School Programming Contest, 2009.
 * @author Chau-Wen Tseng
 */
public class Team {
	static final int maxTeams = 10000;
	static final int maxMemberType = 1000;

	static String memberTypes[] = new String[maxMemberType];
	static int numMemberTypes = 0;

	static int memberNames[];
	static int allMembers = 0;
	static int numTeams = 0;

	static String teamID[] = new String [maxTeams];
	static int members[][];
	static int teamLeader[] = new int [maxTeams];
	static boolean lowestLvl[] = new boolean [maxTeams];
	static double VIdist[] = new double [maxTeams];;


	static int bestSubgroup [][] = new int [maxTeams][];
	static double bestVIdist[] = new double [maxTeams];


	static String format1 = "###0.000000" ;  
	static DecimalFormat fm1 = new DecimalFormat( format1,
			new DecimalFormatSymbols(Locale.US));

	static int memberType(String s) {
		for (int i = 0; i < numMemberTypes; i++) {
			if (s.equals(memberTypes[i]))
				return i;
		}
		// System.err.println("Illegal member type: " + s);
		return -1;
	}

	static String memberName(int t) {
		return memberTypes[t];
	}

	static int teamNum(String ID) {
		for (int i=0; i< numTeams; i++) {
			if (teamID[i].equals(ID)) 
				return i;
		}
		// System.err.println("Can't find team " + ID);
		return -1;
	}

	public static void main(String[] args) throws Exception {
		String s;

		// initialization
		for (int t = 0; t<maxTeams; t++) {
			teamID[t] = null;
			teamLeader[t] = -1;
			lowestLvl[t] = false;
			VIdist[t] = 0.0;
			bestVIdist[t] = 0.0;
			bestSubgroup[t] = null;
		}

		// read input
		Scanner sc = new Scanner( System.in );
		String line = sc.nextLine();
		Scanner lsc = new Scanner(line);

		// 1st line must be list of member types
		numMemberTypes = 0;
		while (lsc.hasNext()) {
			s = lsc.next();
			if (memberType(s) < 0)
				memberTypes[numMemberTypes++] = s;
		}
		
		// initialize member [][]
		members = new int [maxTeams][numMemberTypes];
		for (int t = 0; t<maxTeams; t++) {
			for (int k = 0; k<numMemberTypes; k++) {
				members[t][k] = 0;
			}
		}

		// read in team rosters
		line = sc.nextLine();
		lsc = new Scanner(line);
		int tnum = lsc.nextInt();
		for (int i = 0; i< tnum; i++) {
			line = sc.nextLine();
			lsc = new Scanner(line);
			String ID = lsc.next();
			teamID[i] = ID;
			numTeams++;
			lowestLvl[i] = true;
			// read in list of team members
			while (lsc.hasNext()) {
				s = lsc.next();
				int kType = memberType(s);
				// increment member count for type of Member found
				members[i][kType] += 1;
			}
		}

		/*
		// read in command assignments
		line = sc.nextLine();
		lsc = new Scanner(line);
		int cnum = lsc.nextInt();
		
		// System.err.println("Command Assignments = " + cnum);
		
		for (int i = tnum; i< cnum+tnum; i++) {
			line = sc.nextLine();
			
			// System.err.println("Command Assignments " + i + " = " + line);

			lsc = new Scanner(line);
			String ID = lsc.next();
			teamID[i] = ID;
			numTeams++;
			// read in list of subordinate commanders
			while (lsc.hasNext()) {
				s = lsc.next();
				int sub = teamNum(s);
				// remember current team is leader
				teamLeader[sub]= i;

				// add members supervised by sub to current team
				for (int k = 0; k<numMemberTypes; k++) {
					members[i][k] += members[sub][k];
				}

			}
		}

		computeVIdist();
		computeBestVIdist();
		*/
		computeVIdist();

		// diagnostic
		for (int i = 0; i<numTeams; i++) {
			// displayTeam(i);			
		}
		
		// handle commands one line at a time
		while (sc.hasNextLine()) {
			line = sc.nextLine();
			lsc = new Scanner(line);
			while (lsc.hasNext()) {
				s = lsc.next();
				int t;
				if (s.equals("ROSTER")) {
					s = lsc.next();
					if (s.equals("ALL")) {
						tnum = numTeams-1;
					} 
					else {
						tnum = teamNum(s);
					}
					System.out.print(s + " ");
					for (int k = 0; k<numMemberTypes; k++) {
						System.out.print(members[tnum][k] + " " + memberName(k));
					}
					System.out.println();
				} 
				else if (s.equals("VSCORE")) {
					s = lsc.next();
					tnum = teamNum(s);
					System.out.println(s + " "+ fm1.format(VIdist[tnum]));
				} 
				else if (s.equals("COMMANDER")) {
					s = lsc.next();
					tnum = teamNum(s);
					System.out.println(s + " > " + teamID[teamLeader[tnum]]);
				} 
				else if (s.equals("COMMANDTEAM")) {
					int group[] = new int [maxTeams];
					int groupSize = 0;
					// read in remainder of group
					while (lsc.hasNext()) {
						s = lsc.next();
						tnum = teamNum(s);
						if (tnum < 0) {
							System.out.println("ILLEGAL COMMANDTEAM MEMBER" + s);
						}
						else {
							group[groupSize++] = tnum;
						}
					} 

					// check whether legal group
					if (checkGroup(groupSize, group)) {
						System.out.println("YES");
					} else {
						System.out.println("NO");
					}

				} 
				else if (s.equals("BESTCOMMAND")) {
					tnum = numTeams-1;
					System.out.print(fm1.format(VIdist[numTeams - 1]));
					for (int i = 0; i< bestSubgroup[tnum].length; i++) {
						System.out.print(" "+teamID[bestSubgroup[tnum][i]]);
					}
					System.out.println();
				}
			}
		}
	}

	static boolean checkGroup(int gsize, int group[]) {
		for (int i = 0; i<numTeams; i++) {
			// not leaf, skip
			if (!lowestLvl[i])
				continue;

			// if leaf, ensure exactly single group member in chain of command
			int found = 0;
			int commander = i;	// check self as well

			while (commander >= 0) {
				for (int g = 0; g < gsize; g++) {
					if (group[g] == commander)
						found++;
				}
				commander = teamLeader[commander];
			}
			if (found != 1)
				return false;
		}
		return true;
	}


	static void computeVIdist() {
		// count up total number of members from leaves
		allMembers = 0;

		for (int i = 0; i<numTeams; i++) {
			if (lowestLvl[i]) {
				for (int k = 0; k<numMemberTypes; k++) {
					allMembers += members[i][k];
				}
			}
		}

		// compute variation of information distance for each team
		for (int i = 0; i<numTeams; i++) {
			VIdist[i] = teamVIdist(allMembers, i);		 
		}
	}

	static void computeBestVIdist() {
		if (allMembers == 0)
			computeVIdist();

		int subgroup[] = new int [maxTeams];
		int groupSize;

		for (int i = 0; i<numTeams; i++) {
			if (lowestLvl[i]) {
				// for leaf, just use local VIdist
				bestVIdist[i] = VIdist[i];
				bestSubgroup[i] = new int [1];
				bestSubgroup[i][0] = i;
			} else {
				// for commander, best is either self or SUM best(children)
				double childrenVIdist = 0.0;
				groupSize = 0;

				// find children
				for (int c = 0; c<numTeams; c++) {
					if (teamLeader[c] == i) {
						childrenVIdist += bestVIdist[c];
						for (int g = 0; g < bestSubgroup[c].length; g++) {
							subgroup[groupSize++] = bestSubgroup[c][g];
						}
					}
				}

				// update bestVIdist & bestSubgroup for team
				if (VIdist[i] < childrenVIdist) {
					// commander better than children
					bestVIdist[i] = VIdist[i];
					bestSubgroup[i] = new int [1];
					bestSubgroup[i][0] = i;
				} else {
					// children are better than commander
					bestVIdist[i] = childrenVIdist;
					bestSubgroup[i] = new int [groupSize];
					for (int g = 0; g<groupSize; g++)
						bestSubgroup[i][g] = subgroup[g];
				}
			}
		}
	}

	static void teamComposition(int teamNum) {
		for (int i = 0; i<numTeams; i++) {
			displayTeam(i);			
		}
	}

	static void displayTeam (int num) {
		int all = 0;
		for (int k = 0; k<numMemberTypes; k++) {
			all += members[num][k];
		}
		System.out.println("Team "+teamID[num]);

		if (teamLeader[num] > 0) 
			System.out.println("\tLeader = " + teamID[teamLeader[num]]);

		System.out.println("\tEnrollment = "+ all);
		for (int k = 0; k<numMemberTypes; k++) {
			System.out.println("\t" + members[num][k] + "\t" + memberName(k) +  
					" ("+ (100*members[num][k])/all + "%)");
		}

		System.out.println("\tVIdist = "+ fm1.format(VIdist[num]));
		System.out.println("\tBest VIdist = "+ fm1.format(bestVIdist[num]));

		System.out.print("\tBest Subgroup =");
		for (int i = 0; i< bestSubgroup[num].length; i++) {
			System.out.print(" "+teamID[bestSubgroup[num][i]]);
		}
		System.out.println();
	}

	static double teamVIdist(int all, int t) {
		// Quality = q(x) where
		// 		q(x) = p_x*log(p_x) - 2*SUM ( p_xk*log(p_xk) )
		//		p_x = fraction of all members in army on team
		//		p_k = fraction of team members that are of type k
		//		p_xk = p_x * p_k

		int teamSize = 0;
		for (int k = 0; k<numMemberTypes; k++) {
			teamSize += members[t][k];
		}
		double p_x = ((double) teamSize) / all;
		double sum = 0.0;
		for (int k = 0; k<numMemberTypes; k++) {
			int n = members[t][k];
			if (n > 0) {
				double percentage = ((double) n) / teamSize;
				double p_xk = p_x * percentage;
				sum += p_xk * Math.log10(p_xk);
			}
		}
		return (p_x * Math.log10(p_x)) - (2*sum);
	}
}
